package meerkat;

import meerkat.Registry.*;
import meerkat.bulletinboard.BulletinBoardClient;
import meerkat.crypto.DigitalSignature;
import meerkat.protobuf.BulletinBoardAPI.BulletinBoardMessage;
import meerkat.protobuf.BulletinBoardAPI.UnsignedBulletinBoardMessage;
import meerkat.protobuf.Crypto;
import meerkat.protobuf.VoterRegistry.GroupID;
import meerkat.protobuf.VoterRegistry.VoterGroup;
import meerkat.protobuf.VoterRegistry.VoterID;
import meerkat.protobuf.VoterRegistry.VoterInfo;
import meerkat.protobuf.VoterRegistry.VoterRegistryMessage;

import java.io.InputStream;
import java.security.SignatureException;
import java.util.ArrayList;
import java.util.List;

/**
 * TODO : add ability to use DB of certificates
 */

/**
 * Created by Vladimir Eliezer Tokarev on 1/8/2016.
 * Gives the ability to synchronously manage voters information
 */
public class SimpleRegistry  implements VoterRegistry{

    protected DigitalSignature signer;
    protected BulletinBoardClient bulletinBoardClient ;
    protected Crypto.Signature  signature;
    protected InputStream certificateStream;

    public SimpleRegistry(DigitalSignature signer,
                          BulletinBoardClient communicator,
                          InputStream certificateStream) {
        this.signer = signer;
        this.bulletinBoardClient  = communicator;
        this.certificateStream = certificateStream;
    }

    /**
     * Creates BulletinBoardMessage with signed basicMessage and
     *         UnsignedBulletinBoardMessage that contains the basic message
     *
     * @param basicMessage BasicMessage
     * @return BulletinBoardMessage
     */
    private BulletinBoardMessage CreateBulletinBoardMessage(UnsignedBulletinBoardMessage basicMessage) {
        try {
            BulletinBoardMessage.Builder bulletinBoardMessage = BulletinBoardMessage.newBuilder();

            signer.updateContent(basicMessage);
            signature = signer.sign();
            bulletinBoardMessage.setMsg(basicMessage).addSig(signature);

            return bulletinBoardMessage.build();
        } catch (SignatureException e) {
            e.printStackTrace();
            return null;
        }
    }

    public void AddVoter(VoterInfo voterInfo, RegistryCallBack callback) {
        VoterRegistryMessage.Builder wrapper = VoterRegistryMessage.newBuilder().
                setData(voterInfo.toByteString()).setCreationMiliseconds(System.currentTimeMillis());
        UnsignedBulletinBoardMessage.Builder basicMessage =
                UnsignedBulletinBoardMessage.newBuilder().
                        addTag(RegistryTags.ID_TAG + voterInfo.getId().getId())
                        .addTag(RegistryTags.VOTER_ENTRY_TAG)
                        .addTag(RegistryTags.VOTER_DATA_TAG + voterInfo.getInfo())
                        .setData(wrapper.build().toByteString());

        bulletinBoardClient.postMessage(CreateBulletinBoardMessage(basicMessage.build()), new BooleanCallBack(callback));
    }

    public void AddToGroup(VoterGroup voterGroup, RegistryCallBack callback) {
        VoterRegistryMessage.Builder wrapper = VoterRegistryMessage.newBuilder().
                setData(voterGroup.toByteString()).setCreationMiliseconds(System.currentTimeMillis());
        UnsignedBulletinBoardMessage.Builder basicMessage =
                UnsignedBulletinBoardMessage.newBuilder()
                        .addTag(RegistryTags.ID_TAG + voterGroup.getVoterId().getId())
                        .addTag(RegistryTags.GROUP_ID_TAG + voterGroup.getGroupId().getId())
                        .addTag(RegistryTags.ADD_TO_GROUP_TAG)
                        .setData(wrapper.build().toByteString());

        bulletinBoardClient.postMessage(CreateBulletinBoardMessage(basicMessage.build()), new BooleanCallBack(callback));
    }

    public void SetVoted(VoterID voterId, RegistryCallBack callback) {
        VoterRegistryMessage.Builder wrapper = VoterRegistryMessage.newBuilder().
                setData(voterId.toByteString()).setCreationMiliseconds(System.currentTimeMillis());
        UnsignedBulletinBoardMessage.Builder basicMessage =
                UnsignedBulletinBoardMessage.newBuilder()
                        .addTag(RegistryTags.ID_TAG + voterId.getId())
                        .addTag(RegistryTags.VOTE_ACTION_TAG)
                        .setData(wrapper.build().toByteString());

        bulletinBoardClient.postMessage(CreateBulletinBoardMessage(basicMessage.build()), new BooleanCallBack(callback));
    }

    public void GetGroups(GroupID groupID, RegistryCallBack callback) {
        List<String> GroupsActionsTags = new ArrayList<String>() {{
           add(RegistryTags.GROUP_ID_TAG + groupID.getId());
        }};
        bulletinBoardClient.readMessages(CollectionMessagesUtils.GenerateFiltersFromTags(GroupsActionsTags),
                new RelevantDataCallBack(callback, signer, signature, certificateStream));
    }

    public void GetPersonIDDetails(VoterID voterID, RegistryCallBack callback) {
        List<String> GroupsActionsTags = new ArrayList<String>() {{
            add(RegistryTags.ID_TAG + voterID.getId());
            add(RegistryTags.VOTER_ENTRY_TAG);
        }};
        bulletinBoardClient.readMessages(CollectionMessagesUtils.GenerateFiltersFromTags(GroupsActionsTags),
                new RelevantDataCallBack(callback, signer, signature, certificateStream));
    }
}
